@@ -8,34 +8,49 @@ import os
 import logging
 import typing as t
 import typer
-import lief
 
 from rich import print
 
 from fuzzable import generate
 from fuzzable.config import SOURCE_FILE_EXTS
 from fuzzable.cli import print_table, error, export_results
-from fuzzable.analysis import AnalysisBackend, AnalysisMode, DEFAULT_SCORE_WEIGHTS
+from fuzzable.analysis import AnalysisBackend, DEFAULT_SCORE_WEIGHTS
 from fuzzable.analysis.ast import AstAnalysis
 from fuzzable.log import log
 
 from pathlib import Path
 
 app = typer.Typer(
-    help="Framework for Automating Fuzzable Target Discovery with Static Analysis"
+    help="Framework for Automating Fuzzable Target Discovery with Static Analysis",
+    no_args_is_help=True,
+    context_settings=dict(help_option_names=["-h", "--help"]),
+    add_completion=False,
 )
 
 
+@app.callback()
+def global_args(
+    verbosity: int = typer.Option(
+        0,
+        "-v",
+        "--verbosity",
+        help="Sets logging level (2 = debug, 1 = info, 0 = output).",
+    ),
+):
+    """Handles global flags for all commands."""
+    if verbosity == 1:
+        log.setLevel(logging.INFO)
+    elif verbosity == 2:
+        log.setLevel(logging.DEBUG)
+
+
 @app.command()
 def analyze(
     target: Path,
-    mode: t.Optional[str] = typer.Option(
-        "recommend",
-        help="Analysis mode to run under (either `recommend` or `rank`, default is `recommend`)."
-        "See documentation for more details about which to select.",
-    ),
     export: t.Optional[Path] = typer.Option(
         None,
+        "-e",
+        "--export",
         help="Export the fuzzability report to a path based on the file extension."
         "Fuzzable supports exporting to `json`, `csv`, or `md`.",
     ),
@@ -43,38 +58,41 @@ def analyze(
         False,
         help="If set, will also additionally output and/or export ignored symbols.",
     ),
+    include_sym: t.Optional[str] = typer.Option(
+        None,
+        help="Comma-seperated list of symbols to absolutely be considered for analysis.",
+    ),
+    include_nontop: bool = typer.Option(
+        False, help="If set, won't filter out only on top-level function definitions."
+    ),
+    skip_sym: t.Optional[str] = typer.Option(
+        None, help="Comma-seperated list of symbols to skip during analysis."
+    ),
     skip_stripped: bool = typer.Option(
         False,
         help="If set, ignore symbols that are stripped in binary analysis."
         "Will be ignored if fuzzability analysis is done on source code.",
     ),
+    ignore_metrics: bool = typer.Option(
+        True,
+        help="If set, include individual metrics' scores for each function target analyzed.",
+    ),
     score_weights: t.Optional[str] = typer.Option(
         None,
-        help="Reconfigure the weights for MCDA when determining fuzzability.",
-    ),
-    debug: bool = typer.Option(
-        False,
-        help="If set, will be verbose and output debug information.",
+        "-w",
+        "--score-weights",
+        help="Comma-seperated list of reconfigured weights for multi-criteria decision analysis when determining fuzzability.",
     ),
 ):
     """
     Run fuzzable analysis on a single or workspace of C/C++ source files, or a compiled binary.
     """
-    if debug:
-        log.setLevel(logging.DEBUG)
-
-    if not target.is_file() or target.is_dir():
+    if not target.is_file() and not target.is_dir():
         error(f"Target path `{target}` does not exist.")
 
-    try:
-        mode = AnalysisMode[mode.upper()]
-    except KeyError:
-        error(f"Invalid analysis mode `{mode}`. Must either be `recommend` or `rank`.")
-
-    if mode == AnalysisMode.RANK and list_ignored:
-        error("--list_ignored is not needed for the `rank` mode.")
-
+    # parse custom weights and run checks
     if score_weights:
+        log.debug("Reconfiguring score weights for MCDA")
         score_weights = [float(weight) for weight in score_weights.split(",")]
         num_weights = len(DEFAULT_SCORE_WEIGHTS)
         if len(score_weights) != num_weights:
@@ -85,35 +103,88 @@ def analyze(
     else:
         score_weights = DEFAULT_SCORE_WEIGHTS
 
+    # export file format checking
     if export is not None:
-        ext = export.suffix.lower()
-        if ext not in [".json", ".csv", ".md"]:
+        ext = export.suffix.lower()[1:]
+        if ext not in ["json", "csv", "md"]:
             error("--export value must either have `json`, `csv`, or `md` extensions.")
 
-    log.info(f"Starting fuzzable on {target}")
+    # parse symbols to explicitly include for analysis
+    if include_sym:
+        include_sym = [sym for sym in include_sym.split(",")]
+        if len(include_sym) == 0:
+            error(f"--include-sym must include at least one valid function symbol")
+
+        log.debug(f"Parsed symbols to explicitly include for analysis {include_sym}")
+    else:
+        include_sym = []
+
+    # parse symbols to explicitly exclude from analysis
+    if skip_sym:
+        skip_sym = [sym for sym in skip_sym.split(",")]
+        if len(skip_sym) == 0:
+            error(f"--skip-sym must specify a valid function symbol")
+
+        log.debug(f"Parsed symbols to explicitly include for analysis {skip_sym}")
+    else:
+        skip_sym = []
+
+    # check if overlapping symbols
+    if set(skip_sym) & set(include_sym):
+        error(f"Cannot have same symbols in both --include-sym and --skip-sym.")
+
+    log.info(f"Running fuzzability analysis on {target}")
     if target.is_file():
-        run_on_file(target, mode, score_weights, export, list_ignored, skip_stripped)
+        run_on_file(
+            target,
+            export,
+            list_ignored,
+            include_sym,
+            include_nontop,
+            skip_sym,
+            skip_stripped,
+            ignore_metrics,
+            score_weights,
+        )
     elif target.is_dir():
-        run_on_workspace(target, mode, score_weights, export, list_ignored)
+        run_on_workspace(
+            target,
+            export,
+            list_ignored,
+            include_sym,
+            include_nontop,
+            skip_sym,
+            skip_stripped,
+            ignore_metrics,
+            score_weights,
+        )
 
 
 def run_on_file(
     target: Path,
-    mode: AnalysisMode,
-    score_weights: t.List[float],
     export: t.Optional[Path],
     list_ignored: bool,
+    include_sym: t.List[str],
+    include_nontop: bool,
+    skip_sym: t.List[str],
     skip_stripped: bool,
+    ignore_metrics: bool,
+    score_weights: t.List[float],
 ) -> None:
     """Runs analysis on a single source code file or binary file."""
     analyzer: t.TypeVar[AnalysisBackend]
 
     extension = target.suffix
     if extension in SOURCE_FILE_EXTS:
-        analyzer = AstAnalysis([target], mode, score_weights=score_weights)
+        analyzer = AstAnalysis(
+            [target],
+            include_sym=include_sym,
+            include_nontop=include_nontop,
+            score_weights=score_weights,
+        )
     else:
 
-        # Prioritize loading binja as a backend, this may not
+        # Prioritize loading binja as a backend, this will not
         # work if the license is personal/student.
         try:
             import sys
@@ -129,9 +200,11 @@ def run_on_file(
 
             analyzer = BinjaAnalysis(
                 bv,
-                mode,
-                score_weights=score_weights,
+                include_sym=include_sym,
+                include_nontop=include_nontop,
+                skip_sym=skip_sym,
                 skip_stripped=skip_stripped,
+                score_weights=score_weights,
                 headless=True,
             )
 
@@ -145,26 +218,32 @@ def run_on_file(
 
                 analyzer = AngrAnalysis(
                     target,
-                    mode,
+                    include_sym=include_sym,
+                    include_nontop=include_nontop,
+                    skip_sym=skip_sym,
+                    skip_stripped=skip_stripped,
                     score_weights=score_weights,
-                    skip_stripped=True,
                 )
             except ModuleNotFoundError as err:
                 error(f"Unsupported target {target}. Reason: {err}")
 
     log.info(f"Running fuzzable analysis with the {str(analyzer)} analyzer")
     results = analyzer.run()
-    print_table(target, results, analyzer.skipped, list_ignored)
+    print_table(target, results, analyzer.skipped, ignore_metrics, list_ignored)
     if export:
         export_results(export, results)
 
 
 def run_on_workspace(
     target: Path,
-    mode: AnalysisMode,
-    score_weights: t.List[float],
     export: t.Optional[Path],
     list_ignored: bool,
+    include_sym: t.List[str],
+    include_nontop: bool,
+    skip_sym: t.List[str],
+    skip_stripped: bool,  # not used, maybe until we support multiple binaries
+    ignore_metrics: bool,
+    score_weights: t.List[float],
 ) -> None:
     """
     Given a workspace, recursively iterate and parse out all of the source code files
@@ -183,40 +262,51 @@ def run_on_workspace(
         )
 
     analyzer = AstAnalysis(
-        source_files, mode, score_weights=score_weights, basedir=target
+        source_files,
+        include_sym=include_sym,
+        include_nontop=include_nontop,
+        skip_sym=skip_sym,
+        score_weights=score_weights,
+        basedir=target,
     )
     log.info(f"Running fuzzable analysis with the {str(analyzer)} analyzer")
     results = analyzer.run()
-    print_table(target, results, analyzer.skipped, list_ignored)
+    print_table(target, results, analyzer.skipped, ignore_metrics, list_ignored)
     if export:
         export_results(export, results)
 
 
 @app.command()
 def create_harness(
-    target: str,
+    target: Path,
     symbol_name: str = typer.Option(
-        "",
+        ...,
+        "-n",
+        "--symbol_name",
         help="Names of function symbol to create a fuzzing harness to target. Source not supported yet.",
     ),
     out_so_name: t.Optional[Path] = typer.Option(
         None,
+        "-s",
+        "--out_so_name",
         help="Specify to set output `.so` path of a transformed ELF binary for binary targets.",
     ),
     out_harness: t.Optional[Path] = typer.Option(
-        None, help="Specify to set output harness template file path."
-    ),
-    debug: bool = typer.Option(
-        False,
-        help="If set, will be verbose and output debug information.",
+        None,
+        "-o",
+        "--out_harness",
+        help="Specify to set output harness template file path.",
     ),
 ):
-    """Synthesize a AFL++/libFuzzer harness for a given symbol in a target."""
-    if debug:
-        log.setLevel(logging.DEBUG)
+    """Synthesize a AFL++/libFuzzer harness for a given symbol in a binary target."""
 
-    if not symbol_name:
-        error("No --symbol-name specified.")
+    if not target.is_file():
+        error(f"Target path `{target}` does not exist.")
+
+    try:
+        import lief
+    except (RuntimeError, ModuleNotFoundError, ImportError):
+        error("Could not import LIEF library to parse binary.")
 
     # if a binary, check if executable or library. if executable, use LIEF to
     # copy, export the symbol and transform to shared object.
@@ -227,7 +317,6 @@ def create_harness(
         )
 
     # resolve paths appropriately
-    target = Path(target)
     if out_so_name:
         out_so_name = out_so_name.expanduser()
 
