@@ -10,7 +10,7 @@ import typing as t
 from pathlib import Path
 from tree_sitter import Language, Node, Parser
 
-from . import AnalysisBackend, AnalysisMode, Fuzzability, DEFAULT_SCORE_WEIGHTS
+from . import AnalysisBackend, Fuzzability, DEFAULT_SCORE_WEIGHTS
 from ..metrics import CallScore
 from ..log import log
 from ..config import get_project_root, SOURCE_FILE_EXTS
@@ -25,11 +25,13 @@ class AstAnalysis(AnalysisBackend):
     def __init__(
         self,
         target: t.List[str],
-        mode: AnalysisMode,
+        include_sym: t.List[str] = [],
+        include_nontop: bool = False,
+        skip_sym: t.List[str] = [],
         score_weights: t.List[float] = DEFAULT_SCORE_WEIGHTS,
         basedir: t.Optional[Path] = None,
     ):
-        super().__init__(target, mode, score_weights)
+        super().__init__(target, include_sym, include_nontop, skip_sym, score_weights)
 
         log.debug("Building third-party tree-sitter libraries for C/C++ languages")
         Language.build_library(
@@ -66,46 +68,7 @@ class AstAnalysis(AnalysisBackend):
         # first collect ASTs for every function
         log.info("Collecting and parsing ASTs for each function call")
         for filename in self.target:
-
-            # fix up path if a basedir is present
-            if self.basedir:
-                filepath = filename.relative_to(self.basedir)
-            else:
-                filepath = filename
-
-            # if recommend mode, ignore all unit tests
-            if self.mode == AnalysisMode.RECOMMEND and "test" in str(filepath).lower():
-                log.info(f"{filepath} - skipping as it's a potential unit test file")
-                continue
-
-            # switch over language if different language detected
-            extension = filepath.suffix
-            if extension in SOURCE_FILE_EXTS[1:]:
-                self.language = Language(BUILD_PATH, "cpp")
-            else:
-                self.language = Language(BUILD_PATH, "c")
-
-            self.parser.set_language(self.language)
-
-            with open(filename, "rb") as source_file:
-                contents = source_file.read()
-
-            tree = self.parser.parse(contents)
-            # log.debug(tree.root_node.sexp())
-
-            log.debug(f"{filepath} - grabbing function definitions")
-            query = self.language.query(
-                """
-            (function_definition) @capture
-            """
-            )
-
-            # TODO: skip out on `static` calls if recommend mode
-
-            # store mappings for the file
-            log.debug(f"{filepath} - aggregating definition captures")
-            captures = [node for (node, _) in query.captures(tree.root_node)]
-            self.parsed_symbols[filepath] = (captures, contents)
+            self._parse_symbols(filename)
 
         # now analyze each function_definition node
         log.info("Statically analyzing and calculating fuzzability for each call")
@@ -113,10 +76,10 @@ class AstAnalysis(AnalysisBackend):
             nodes = entry[0]
             contents = entry[1]
             for node in nodes:
+                path = f"{filename}:{node.start_point[0]}"
 
-                # first, parse out name to see if we should visit again
                 log.debug(
-                    f"{filename} - attempting to capture function symbol name for the current node AST"
+                    f"Attempting to capture function symbol name for the current node AST at {path}"
                 )
                 query = self.language.query(
                     """
@@ -132,9 +95,9 @@ class AstAnalysis(AnalysisBackend):
                     )
                 except Exception as err:
                     log.warning(
-                        f"{filename} - parsing failed for {node}, reason: {err}"
+                        f"Parsing failed for {node} in {filename}, reason: {err}"
                     )
-                    self.skipped += 1
+                    self.skipped[name] = path
                     continue
 
                 if name in self.visited:
@@ -142,26 +105,68 @@ class AstAnalysis(AnalysisBackend):
                     continue
                 self.visited += [name]
 
-                log.debug(f"{filename} - checking if we should skip analysis for node")
+                log.debug(f"Checking if we should ignore {name}")
                 if self.skip_analysis(name):
-                    self.skipped[name] = f"{filename}:{node.start_point[0]}"
-                    log.debug(f"{filename} - skipping over this one")
+                    self.skipped[name] = path
+                    log.warning(f"Skipping {name} from fuzzability analysis.")
                     continue
 
-                # if recommend mode, filter and run only those that are top-level
-                self.is_top_level = self.is_toplevel_call(node)
-                if self.mode == AnalysisMode.RECOMMEND and not self.is_top_level:
-                    log.debug(
-                        f"{filename} - skipping over node, since it's not top-level for recommended mode"
+                log.debug(f"Checking to see if {name} is a top-level call")
+                self.is_top_level = self.is_toplevel_call(name, node)
+                if not self.include_nontop and not self.is_top_level:
+                    log.warning(
+                        f"Skipping {name} (not top-level) from fuzzability analysis."
                     )
-                    self.skipped[name] = f"{filename}:{node.start_point[0]}"
+                    self.skipped[name] = path
                     continue
 
-                log.debug(f"{filename} - analyzing function target {name}")
+                log.info(f"Starting analysis for function {name}")
                 self.scores += [self.analyze_call(name, node, filename, contents)]
 
         return super()._rank_fuzzability(self.scores)
 
+    def _parse_symbols(self, filename: Path) -> None:
+        """Helper to recover all function implementations from a source target"""
+
+        # fix up path if a basedir is present
+        if self.basedir:
+            filepath = filename.relative_to(self.basedir)
+        else:
+            filepath = filename
+
+        # ignore all unit tests (TODO: enable)
+        if "test" in str(filepath).lower():
+            log.info(f"{filepath} - skipping as it's a potential unit test file")
+            return None
+
+        # switch over language if different language detected
+        extension = filepath.suffix
+        if extension in SOURCE_FILE_EXTS[1:]:
+            self.language = Language(BUILD_PATH, "cpp")
+        else:
+            self.language = Language(BUILD_PATH, "c")
+
+        self.parser.set_language(self.language)
+
+        with open(filename, "rb") as source_file:
+            contents = source_file.read()
+
+        tree = self.parser.parse(contents)
+        # log.debug(tree.root_node.sexp())
+
+        log.debug(f"Grabbing function definitions in {filepath}")
+        query = self.language.query(
+            """
+        (function_definition) @capture
+        """
+        )
+
+        log.debug(f"Aggregating definition captures in {filepath}")
+
+        # store function definition mappings for the file
+        captures = [node for (node, _) in query.captures(tree.root_node)]
+        self.parsed_symbols[filepath] = (captures, contents)
+
     def analyze_call(
         self, name: str, func: Node, filename: str, contents: bytes
     ) -> CallScore:
@@ -178,9 +183,9 @@ class AstAnalysis(AnalysisBackend):
         )
 
     def skip_analysis(self, name: str) -> bool:
-        """
-        WIP
-        """
+        """Handles parsing edge cases that yield weird function nodes"""
+        if super().skip_analysis(name):
+            return True
 
         # name parsed is primitive type, skip
         if name in ["void", "int", "char"]:
@@ -192,14 +197,16 @@ class AstAnalysis(AnalysisBackend):
 
         return False
 
-    def is_toplevel_call(self, target: str) -> bool:
+    def is_toplevel_call(self, name: str, node: Node) -> bool:
+        """
+        Function implementation should not be static, and has no parent
+        callers.
         """
-        Check if node is a callee of any other function nodes, and if not is considered
-        a top level call
 
-        TODO: can this be more performant and pythonic?
-        """
-        log.debug(f"{target} - checking if toplevel call")
+        # ignore if static function
+        # TODO: deal with other edge cases and potential macro aliases
+        if node.children[0].type == "storage_class_specifier":
+            return False
 
         # get call_expressions for each function name
         for _, entry in self.parsed_symbols.items():
@@ -219,7 +226,7 @@ class AstAnalysis(AnalysisBackend):
                     call_name = contents[capture.start_byte : capture.end_byte].decode(
                         "utf8"
                     )
-                    if call_name == target:
+                    if call_name == name:
                         return False
 
         return True
