@@ -14,8 +14,10 @@ from pathlib import Path
 import binaryninja
 import binaryninja.log as log
 import binaryninja.interaction as interaction
+import binaryninja.demangle as demangle
 
 from binaryninja import BinaryView
+from binaryninja.architecture import Architecture
 from binaryninja.function import Function
 from binaryninja.lowlevelil import LowLevelILReg
 from binaryninja.enums import LowLevelILOperation, SymbolType
@@ -23,9 +25,8 @@ from binaryninja.plugin import BackgroundTaskThread
 from binaryninja.settings import Settings
 
 from .. import generate
-from . import AnalysisBackend, AnalysisMode, Fuzzability, DEFAULT_SCORE_WEIGHTS
-from ..metrics import CallScore
-from ..cli import COLUMNS, CSV_HEADER
+from . import AnalysisBackend, Fuzzability, DEFAULT_SCORE_WEIGHTS
+from ..metrics import CallScore, METRICS
 
 
 class _BinjaAnalysisMeta(type(AnalysisBackend), type(BackgroundTaskThread)):
@@ -40,12 +41,22 @@ class BinjaAnalysis(
     def __init__(
         self,
         target: BinaryView,
-        mode: AnalysisMode,
+        include_sym: t.List[str] = [],
+        include_nontop: bool = False,
+        skip_sym: t.List[str] = [],
+        skip_stripped: bool = False,
         score_weights: t.List[float] = DEFAULT_SCORE_WEIGHTS,
         headless: bool = False,
-        skip_stripped: bool = False,
     ):
-        AnalysisBackend.__init__(self, target, mode, score_weights)
+        AnalysisBackend.__init__(
+            self,
+            target,
+            include_sym,
+            include_nontop,
+            skip_sym,
+            skip_stripped,
+            score_weights,
+        )
         BackgroundTaskThread.__init__(
             self, "Finding fuzzable targets in current binary view"
         )
@@ -61,21 +72,24 @@ class BinjaAnalysis(
 
         log.log_info(f"Starting fuzzable analysis over {len(funcs)} symbols in binary")
         for func in funcs:
-            name = func.name
 
-            # if name in self.visited:
-            #    continue
-            # self.visited += [name]
+            # demangle the symbol name
+            _, name = demangle.demangle_ms(Architecture["x86_64"], func.name)
+            name = demangle.get_qualified_name(name)
+            addr = str(hex(func.address_ranges[0].start))
 
             log.log_debug(f"Checking to see if we should ignore {name}")
-            if self.mode == AnalysisMode.RECOMMEND and self.skip_analysis(func):
-                self.skipped[name] = str(hex(func.address_ranges[0].start))
+            if self.skip_analysis(func):
+                log.log_warn(f"Skipping {name} from fuzzability analysis.")
+                self.skipped[name] = addr
                 continue
 
-            # if recommend mode, filter and run only those that are top-level
             log.log_debug(f"Checking to see if {name} is a top-level call")
-            if self.mode == AnalysisMode.RECOMMEND and not self.is_toplevel_call(func):
-                self.skipped[name] = str(hex(func.address_ranges[0].start))
+            if not self.include_nontop and self.is_toplevel_call(func):
+                log.log_warn(
+                    f"Skipping {name} (not top-level) from fuzzability analysis."
+                )
+                self.skipped[name] = addr
                 continue
 
             log.log_info(f"Starting analysis for function {name}")
@@ -87,8 +101,10 @@ class BinjaAnalysis(
 
         # if headless, handle displaying results back
         if not self.headless:
-            csv_result = CSV_HEADER
-            csv_result = ", ".join([f'"{column}"' for column in COLUMNS])
+            csv_result = ",".join([metric.identifier for metric in METRICS])
+
+            columns = [metric.friendly_name for metric in METRICS]
+            csv_result = ", ".join([f'"{column}"' for column in columns])
 
             # TODO: reuse rich for markdown
             markdown_result = f"""# Fuzzable Targets
@@ -107,7 +123,6 @@ __Top Fuzzing Contender:__ [{ranked[0].name}](binaryninja://?expr={ranked[0].nam
 | Function Signature | Location          | Fuzzability Score | Fuzz-Friendly Name | Risky Data Sinks | Natural Loops | Cyclomatic Complexity | Coverage Depth |
 |--------------------|-------------------|-------------------|--------------------|------------------|---------------|-----------------------|----------------|
 """
-
             for score in ranked:
                 markdown_result += score.binja_markdown_row
                 csv_result += score.csv_row
@@ -149,7 +164,10 @@ __Top Fuzzing Contender:__ [{ranked[0].name}](binaryninja://?expr={ranked[0].nam
     def skip_analysis(self, func: Function) -> bool:
         name = func.name
         symbol = func.symbol.type
-        log.log_debug(f"{name} - {symbol}")
+        log.log_debug(f"Checking if we should skip {name} ({symbol})")
+
+        if super().skip_analysis(name):
+            return True
 
         # ignore imported functions from other libraries, ie glibc or win32api
         if symbol in [
@@ -162,17 +180,6 @@ __Top Fuzzing Contender:__ [{ranked[0].name}](binaryninja://?expr={ranked[0].nam
             log.log_debug(f"{name} is an import, skipping")
             return True
 
-        # ignore targets with patterns that denote some type of profiling instrumentation, ie stack canary
-        # if name.startswith("__"):
-        #    log.log_debug(f"{name} is potentially instrumentation, skipping")
-        #    return True
-
-        # if set, ignore all stripped functions for faster analysis
-        # if ("sub_" in name) and Settings().get_bool("fuzzable.skip_stripped"):
-        if "sub_" in name and self.mode == AnalysisMode.RECOMMEND:
-            log.log_debug(f"{name} is stripped, skipping")
-            return True
-
         return False
 
     def is_toplevel_call(self, target: Function) -> bool:
@@ -180,12 +187,12 @@ __Top Fuzzing Contender:__ [{ranked[0].name}](binaryninja://?expr={ranked[0].nam
 
     def risky_sinks(self, func: Function) -> int:
         """
-        For each parameter in the function, determine if it flows into a known risky
-        function call.
+        Find references of known insecure/risky calls, and check to see if there is an
+        argument in the function call that flows into it. Will treat the current function
+        target under an interprocedural analysis.
         """
 
         risky_sinks = 0
-
         visited = []
 
         # visit all other calls with depth-first search until we reach a risky sink
@@ -278,13 +285,16 @@ __Top Fuzzing Contender:__ [{ranked[0].name}](binaryninja://?expr={ranked[0].nam
         return num_edges - num_blocks + 2
 
 
-def run_fuzzable_recommend(view) -> None:
-    task = BinjaAnalysis(view, AnalysisMode.RECOMMEND)
-    task.start()
-
-
-def run_fuzzable_rank(view) -> None:
-    task = BinjaAnalysis(view, AnalysisMode.RANK)
+def run_fuzzable(view) -> None:
+    settings = Settings()
+    task = BinjaAnalysis(
+        view,
+        include_sym=settings.get_array("fuzzable.include_sym"),
+        include_nontop=settings.get_bool("fuzzable.include_nontop"),
+        skip_sym=settings.get_array("fuzzable.skip_sym"),
+        skip_stripped=settings.get_bool("fuzzable.skip_stripped"),
+        score_weights=settings.get_array("fuzzable.score_weights"),
+    )
     task.start()
 
 
